#include <math.h>

#include "postgres.h"
#include "executor/spi.h"	/* this is what you need to work with SPI */
#include "commands/trigger.h"	/* -"- and triggers */

/*
 * Structure used to accumulate statistics.
 *
*/

typedef struct {
    float8	ssq;
    float8	s;
    float8      n;
} ctx_type;

static text* _stats_collect(text *ctxPtr, float8 f2);

text*
stats_collectf8(text *ctxPtr, float8 *v)
{
	return _stats_collect(ctxPtr, *v);
}

text*
stats_collectf4(text *ctxPtr, float4 *v)
{
        return _stats_collect(ctxPtr, *v);
}

text*
stats_collecti4(text *ctxPtr, int4 v)
{
        return _stats_collect(ctxPtr, v);
}

static text*
_stats_collect(text *ctxPtr, float8 f2)
{
	ctx_type	*ctx;
	int4		*result;
	int4		len;

	//
	// If this is the first time, allocate the context.
	//
	if ( VARSIZE(ctxPtr) <= VARHDRSZ ) {
		len = VARHDRSZ + sizeof(ctx_type);
		ctxPtr = (text*)palloc(len);
		VARSIZE(ctxPtr) = len;
		ctx = (ctx_type*)VARDATA(ctxPtr);
		ctx->ssq = 0;
		ctx->s = 0;
		ctx->n = 0;
	} else {
		ctx = (ctx_type*)VARDATA(ctxPtr);
	};

	//
	// Collect the stats we need.
	//
	ctx->ssq += f2 * f2;
	ctx->s += f2;
	ctx->n++;

	//
	// Return the context pointer.
	//
	return ctxPtr;
}

//
// Calculate sdev
//

float8*
stats_variance(text *ctxPtr, void *p)
{
        ctx_type        *ctx;
        float8          *result;
        float8          avg;

	result = (float8*)palloc(sizeof(float8));

        if (ctxPtr == NULL) {
                (*result) = 0;
        } else {
                ctx = (ctx_type*)VARDATA(ctxPtr);
                avg = ctx->s / ctx->n;
                (*result) = (ctx->ssq - 2*avg*ctx->s)/ctx->n + avg*avg;
                pfree(ctxPtr);
        };

        return result;
}

float8*
stats_sdev(text *ctxPtr, void *p)
{
        float8  *resTmp;

        resTmp = stats_variance(ctxPtr,p);
        (*resTmp) = sqrt(*resTmp);

        return resTmp;
}


